local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local string_format = string.format
local SetCVar = SetCVar
local GetCVar = GetCVar
local GetCVarBool = GetCVarBool
local GetRefreshs = GetRefreshs
local pairs = pairs
local get_cvar_number = ControlPanel_Options.get_cvar_number

local function tablefind(array,key)
	local i = 1
	for i = 1,#array do
		if key == array[i] then
			return i
		end
	end
	return i
end

local string_upper = string.upper
local RestartGx = RestartGx
local languages = {deDE="deDE",enUS="enUS",esES="esES",esMX="esMX",frFR="frFR",itIT="itIT",koKR="koKR",ptBR="ptBR",ruRU="ruRU",zhCN="zhCN",zhTW="zhTW"}
local allow_global_operations = false
local graphicsAPIs = { GetGraphicsAPIs() }
local resolutions = { GetScreenResolutions() }

local function get_refresh_rates()
	local refs = { GetRefreshRates() }
	local ret = {}
	local i
	local n = #refs
	for i=1,n,2 do
		ret[string_format("%d/%d",refs[i],refs[i+1])]=string_format("%.0fHz",refs[i]/refs[i+1])
	end
	return ret
end

local refresh_rates = get_refresh_rates()

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end
local function SetCVar_Global(key,value)
	if allow_global_operations == true then
		SetCVar(key,value)
	end
	allow_global_operations = false
end
local Global =
{
	name = L["Global"],
	desc = L["Requires a client restart."].."\n\n"..L["ControlPanel won't manage these options."],
	type = "group",
	args =
	{
		Allow_Once =
		{
			order = get_order(),
			name = L["Allow once"],
			desc = L["Prevent misoperations on Global Settings"],
			type = "toggle",
			set = function(info,val)
				allow_global_operations = val
			end,
			get  = function(info)
				return allow_global_operations
			end,
		},
		Display =
		{
			order = get_order(),
			name = L["Display"],
			desc = L["Requires a restart of the game's graphic subsystem"],
			type = "group",
			args =
			{
				gxFullscreenResolution =
				{
					order = get_order(),
					name = L["Resolution"],
					desc = L["Higher resolution will result in increased clarity, but this greatly affects performance. Choose a resolution that matches the aspect ratio of your monitor."],
					type = "select",
					values = resolutions,
					set = function(info,val)
						SetCVar_Global("gxFullscreenResolution",resolutions[val])
					end,
					get = function(info)
						return GetCurrentResolution()
					end,
				},
				gxRefresh =
				{
					order = get_order(),
					name = L["Refresh Rate"],
					type = "select",
					values = refresh_rates,
					set = function(info,val)
						SetCVar_Global("gxRefresh",val)
					end,
					get = function(info)
						return GetCVar("gxRefresh")
					end,
				},
				gxWindow =
				{
					order = get_order(),
					name = L["Windowed"],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxWindow",val)
					end,
					get = function(info)
						return GetCVarBool("gxWindow")
					end,
				},
				gxMaximize =
				{
					order = get_order(),
					name = L["Fullscreen"],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxMaximize",val)
					end,
					get = function(info)
						return GetCVarBool("gxMaximize")
					end,
				},
				gxVSync =
				{
					order = get_order(),
					name = L["Vertical Sync"],
					desc = L["Synchronizes your framerate to some fraction of your monitor's refresh rate. Enable this if you see excessive screen tearing in game."],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxVSync",val)
					end,
					get = function(info)
						return GetCVarBool("gxVSync")
					end,
				},

				gxTripleBuffer =
				{
					order = get_order(),
					name = L["Triple Buffering"],
					desc = L["Enables triple buffering of frames when vertical sync is enabled. Selecting triple buffering may even out frame rate spikes, but may also cause slight input lag."],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxTripleBuffer",val)
					end,
					get = function(info)
						return GetCVarBool("gxTripleBuffer")
					end,
				},
				gxFixLag =
				{
					order = get_order(),
					name = L["Reduce Input Lag"],
					desc = L["Enabling this reduces user interface lag, but may drastically reduce frame rates."],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxFixLag",val)
					end,
					get = function(info)
						return GetCVarBool("gxFixLag")
					end,
				},
				gxCursor = 
				{
					order = get_order(),
					name = L["Hardware Cursor"],
					desc = L["Enable this option for a more responsive cursor unless you have cursor problems."],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global("gxCursor",val)
					end,
					get = function(info)
						return GetCVarBool("gxCursor")
					end,
				},
				gxApi =
				{
					order = get_order(),
					name = L["Graphics API"],
					desc = L["Requires a client restart."],
					type = "select",
					values = graphicsAPIs,
					set = function(info,val)
						SetCVar_Global("gxApi",graphicsAPIs[val])
					end,
					get = function(info)
						return tablefind(graphicsAPIs,GetCVar("gxApi"))
					end,
				},
				violenceLevel =
				{
					order = get_order(),
					name = L["Violence Level"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					set = function(info,val)
						SetCVar_Global("violenceLevel",val)
					end,
					get = function(info)
						return get_cvar_number("violenceLevel")
					end,
				},
				restart_graphic_subsystem =
				{
					order = get_order(),
					name = L["Restart graphics subsystem"],
					type = "execute",
					func = function(info)
						RestartGx()
					end,
					width = "full"
				}
			}
		},
		Langauges =
		{
			order = get_order(),
			name = L["Langauges"],
			type = "group",
			args =
			{
				portal = 
				{
					order = get_order(),
					name = string_format(L["Portal : %s"],string_upper(GetCVar("portal"))),
					type = "description",
					fontSize = "large",
					width = "full",
				},
				textLocale =
				{
					order = get_order(),
					name = L["Text"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("textLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("textLocale")
					end,
				},
				audioLocale =
				{
					order = get_order(),
					name = L["Audio"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("audioLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("audioLocale")
					end,
				},
				installLocale =
				{
					order = get_order(),
					name = L["Install"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("installLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("installLocale")
					end,
				},
			},
		},
	}
}

ControlPanel_Options:push("Global",Global)
