local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local string_format = string.format
local SetCVar = SetCVar

local get_cvar_number = ControlPanel_Options.get_cvar_number
local get_cvar_bool = ControlPanel_Options.get_cvar_bool

local GetCVarBool = GetCVarBool

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end

local Interface =
{
	name = L["Interface"],
	desc = L["ControlPanel won't manage these options."],
	type = "group",
	args =
	{

		movieSubtitle = 
		{
			order = get_order(),
			name = L["Cinematic Subtitles"],
			type = "toggle",
			set = function(info,val)
				SetCVar("movieSubtitle",val)
			end,
			get = function(info)
				return get_cvar_bool("movieSubtitle")
			end,
		},

		Camera =
		{
			name = L["Camera"],
			desc = L["These options affect whether certain UI and character elements are hidden or displayed."],
			type = "group",
			args =
			{
				Distance =
				{
					order = get_order(),
					name = L["Distance"],
					type = "group",
					args =
					{
						cameraDistanceMaxFactor =
						{
							order = get_order(),
							name = L["Max Factor"],
							type = "range",
							min =1,
							max = 1.9,
							set = function(info,val)
								SetCVar("cameraDistanceMaxFactor",val)
							end,
							get = function(info)
								return get_cvar_number("cameraDistanceMaxFactor")
							end,
						},
						cameraDistanceMoveSpeed =
						{
							order = get_order(),
							name = L["Move Speed"],
							type = "range",
							min =  0,
							max = 50,
							set = function(info,val)
								SetCVar("cameraDistanceMoveSpeed",val)
							end,
							get = function(info)
								return get_cvar_number("cameraDistanceMoveSpeed")
							end,
						}
					}
				}
			}
		},
		Synchronize =
		{
			name = L["Synchronize"],
			desc = L["Whether client settings should be stored on the server"],
			type = "group",
			args =
			{
				synchronizeSettings = 
				{
					name = L["Settings"],
					type = "toggle",
					set = function(info,val)
						SetCVar("synchronizeSettings",val)
					end,
					get = function(info)
						return GetCVarBool("synchronizeSettings")
					end,
				},
				synchronizeConfig = 
				{
					name = L["Config"],
					type = "toggle",
					set = function(info,val)
						SetCVar("synchronizeConfig",val)
					end,
					get = function(info)
						return GetCVarBool("synchronizeConfig")
					end,
				},
				synchronizeBindings = 
				{
					name = L["Bindings"],
					type = "toggle",
					set = function(info,val)
						SetCVar("synchronizeBindings",val)
					end,
					get = function(info)
						return GetCVarBool("synchronizeBindings")
					end,
				},
				synchronizeMacros = 
				{
					name = L["Macros"],
					type = "toggle",
					set = function(info,val)
						SetCVar("synchronizeMacros",val)
					end,
					get = function(info)
						return GetCVarBool("synchronizeMacros")
					end,
				},
				synchronizeChatFrames = 
				{
					name = L["Chat Frames"],
					type = "toggle",
					set = function(info,val)
						SetCVar("synchronizeChatFrames",val)
					end,
					get = function(info)
						return GetCVarBool("synchronizeChatFrames")
					end,
				},
				forceEnglishNames = 
				{
					order = -1,
					name = L["Force English Names"],
					type = "toggle",
					set = function(info,val)
						SetCVar("forceEnglishNames",val)
					end,
					get = function(info)
						return GetCVarBool("forceEnglishNames")
					end,
				},
			}
		}
	}
}

ControlPanel_Options:push("Interface",Interface)